//Houston Swamp Garden Soil Temperature and Moisture Datalogger Project

//Libraries
#include <Adafruit_CC3000.h>
#include <Adafruit_CC3000_Server.h>
#include <ccspi.h>
#include <OneWire.h>
#include <DallasTemperature.h>
#include "Wire.h"                
#include <SD.h>
#include <SPI.h>
#include <DS3234.h>
#include <string.h>
#include "DHT.h"
                
//Light sensor
int photocellPin = 8;     // the cell and 10K pulldown are connected to a0
int photocellReading;     // the analog reading from the analog resistor divider

//Humidity/Temp DHT22
#define DHTPIN 8
#define DHTTYPE DHT22
DHT dht(DHTPIN, DHTTYPE);

/////////////OneWire SetUp
//Data wire is plugged into pin 3 on the Arduino
  #define ONE_WIRE_BUS 3

// Setup oneWire instance to communicate with OneWire devices
  OneWire oneWire(ONE_WIRE_BUS);

// Pass oneWire reference to Dallas Temperature
  DallasTemperature sensors(&oneWire);
  
//Hex addresses of Temp probes on OneWire
  byte Thermo1[8] = {0x28, 0x23, 0x12, 0x15, 0x06, 0x00, 0x00, 0x66};
  byte Thermo2[8] = {0x28, 0x33, 0x6C, 0x15, 0x06, 0x00, 0x00, 0xF2};
  byte Thermo3[8] = {0x28, 0x17, 0x7C, 0x15, 0x06, 0x00, 0x00, 0xE4};
  byte Thermo4[8] = {0x28, 0x86, 0x50, 0x15, 0x06, 0x00, 0x00, 0xBB};
  byte Thermo5[8] = {0x28, 0x4D, 0x66, 0x15, 0x06, 0x00, 0x00, 0xC4};
  byte Thermo6[8] = {0x28, 0xC2, 0x3C, 0x15, 0x06, 0x00, 0x00, 0x22};
  byte Thermo7[8] = {0x28, 0x52, 0x1A, 0x15, 0x06, 0x00, 0x00, 0xF7};
  byte Thermo8[8] = {0x28, 0xAC, 0x32, 0x15, 0x06, 0x00, 0x00, 0x50}; 

float tempC,Temp1,Temp3,Temp5,Temp7,Temp9,Temp11,TempT,AirTemp; 

///////////////////////////////////////microSD card pins and info
char filename[] = "Log.CSV";
File myFile;

/* 
Arduino MEGA microSD and cc3000 SPI pins 
   MISO/DO = 50
   MOSI/DI = 51
   CLK     = 52
*/
int SD_CS = 42;
int WiFi_CS = 26;

////////////////////////////////////// WiFi CC3000 Set Up
#define ADAFRUIT_CC3000_IRQ   2
#define ADAFRUIT_CC3000_VBAT  27
#define ADAFRUIT_CC3000_CS    26
Adafruit_CC3000 cc3000 = Adafruit_CC3000(ADAFRUIT_CC3000_CS, ADAFRUIT_CC3000_IRQ, ADAFRUIT_CC3000_VBAT,SPI_CLOCK_DIVIDER); 

#define WLAN_SSID       "xxxxxx"
#define WLAN_PASS       "xxxxxxx"
#define WLAN_SECURITY   WLAN_SEC_WPA2
uint8_t dhcpTimeout;

////////////////////////////////////// Ubidots parameters
#define WEBSITE "things.ubidots.com"
#define token "xxxx"  
#define idvariable1 "xxxx"
#define idvariable2 "xxxx"
#define idvariable3 "xxxx"
#define idvariable4 "xxxx"
#define idvariable5 "xxxx"
#define idvariable6 "xxxx"
#define idvariable7 "xxxx"

uint32_t ip;
String webData1;
String webData2;
String webData3;
String webData4;
String webData5;
String webData6;
String webData7;
uint8_t stringLength1 = 0;
uint8_t stringLength2 = 0;
uint8_t stringLength3 = 0;
uint8_t stringLength4 = 0;
uint8_t stringLength5 = 0;
uint8_t stringLength6 = 0;
uint8_t stringLength7 = 0;


void setup()
{
//wake up serial port
  Serial.begin(9600);

//wake up DHt22
  dht.begin();

//wake up temperature probes  
  sensors.begin();
  
//wake up RTC and config pins (MOSI,MISO,CLK,SS)
  RTC.configure(37,34,35,36);
  
//set SPI pin output for microSD (42) 
  pinMode(42, OUTPUT);
  
//uncomment if needed to set clock: day(1-31), month(1-12), year(2 or 4 digits) , hour(0-23), minute(0-59), second(0-59)  
//RTC.setDateTime(6, 1, 2015, 20, 46, 30);

//Wait for newly started system to stabilize
  delay(500);
  
//Interrupt WiFi SPI CS pin so SD can talk  
  digitalWrite( WiFi_CS, HIGH );
  digitalWrite( SD_CS, LOW );

//check if microSD card is present
    if (!SD.begin(SD_CS)) 
  {
    Serial.println("No SD?");
    return;
  }
    Serial.println("SD OK");

//create SD file header
myFile = SD.open(filename, FILE_WRITE);
  if (myFile)
  {
//data header
    String header = "Date & Time, Light, %Humidity, AirTemp, Bed 1, , Bed 3, , Bed 5, , Bed 7, , Bed 9, , Bed 11/12, , Tomatoes, , Average Temp, , Avergae Moisture,";
    myFile.println(header);
    myFile.close();
    Serial.println(header);
  }
  else
  {
    Serial.println("Can't open log file");
  }

//Interrupt microSD SPI CS pin so WiFi can talk  
  digitalWrite( SD_CS, HIGH );
  digitalWrite( WiFi_CS, LOW );
  
//Start up WiFi CC3000
  cc3000.begin();
  Serial.println(F("Initializing WiFi..."));
  if (!cc3000.begin())
  {
    Serial.println(F("WiFi failed!"));
    while(1);
  }
  Serial.println("Succeeded");
  
//Reserving data string for Ubidots
  webData1.reserve(180);
  webData2.reserve(180);
  webData3.reserve(180);
  webData4.reserve(180);
  webData5.reserve(180);
  webData6.reserve(180);
  webData7.reserve(180);

//Connect to WiFi network
  beginning: //goto if connection fails
  
  Serial.print(F("Attempting to connect to "));
  Serial.println(WLAN_SSID);
  cc3000.connectToAP(WLAN_SSID, WLAN_PASS, WLAN_SECURITY);
  Serial.println(F("WiFi Connected! 1"));
  
//Wait for DHCP
  Serial.println(F("Requesting DHCP. 1"));
  while (!cc3000.checkDHCP())
  {
    delay(500);
    Serial.print(F("."));
    dhcpTimeout ++;
    if (dhcpTimeout == 60) 
    {
      Serial.println(F(""));
      Serial.println("DHCP Timeout! Attempting to reconnect. 1");
      dhcpTimeout = 0;
      goto beginning;
    }
  }
  Serial.println(F(""));
  Serial.println(F("DHCP complete. 1"));

//Get website IP & print it
  ip = 0;
  Serial.print(WEBSITE); 
  Serial.print(F(" -> "));
  while (ip == 0) 
  {
    if (!cc3000.getHostByName(WEBSITE, &ip)) 
    {
      Serial.println(F("Couldn't resolve!"));
    }
    delay(500);
  }
  cc3000.printIPdotsRev(ip);
  Serial.println("");
  
//Shut up both microSD and WiFi SPI CS pins
  digitalWrite( SD_CS, HIGH );
  digitalWrite( WiFi_CS, HIGH );

}



void loop() 
{

//DHT22 checkout
  float h = dht.readHumidity();
  if (isnan(h)) 
  {
    Serial.println("DHT22 Failed!");
    return;
  }
  
//Call readings from OneWire addresses
  sensors.requestTemperatures();

//Light sensor and values
  photocellReading = analogRead(photocellPin);  
 
//Rename One Wire Temp probes for string
  AirTemp = sensorValue(Thermo1);
  Temp1 = sensorValue(Thermo2);  
  Temp3 = sensorValue(Thermo3);
  Temp5 = sensorValue(Thermo4);
  Temp7 = sensorValue(Thermo5);
  Temp9 = sensorValue(Thermo6);
  Temp11 = sensorValue(Thermo7);
  TempT = sensorValue(Thermo8);

//Calculate averages for Ubidots string  
  int AvgTemp = (((Temp1) + (Temp3) + (Temp5) + (Temp7) + (Temp9) + (Temp11)) / 6);
  int AvgMoist = (((analogRead(1)) + (analogRead(2)) + (analogRead(3)) + (analogRead(4)) + (analogRead(5)) + (analogRead(6))) / 6);

/////////////////////Print out Temperature, Moisture and light data to serial monitor. For troubleshooting and can be commeneted out.
  Serial.println("Date-Time: ");
  Serial.println(RTC.readDateTime());

  Serial.println("% Humidity: ");
  Serial.println(h);
  
  Serial.print("Light sensor = ");
  Serial.print(photocellReading);     
// Light threshholds, qualitatively determined
  if (photocellReading < 10) {
    Serial.println(" - Dark");
  } else if (photocellReading < 200) {
    Serial.println(" - Dusk");
  } else if (photocellReading < 500) {
    Serial.println(" - DayLight");
  } else if (photocellReading < 800) {
    Serial.println(" - Light is on.");
  } else {
    Serial.println(" - on Fire!");
  } 
  
  Serial.print("      AirTemp = "); 
  Serial.println(AirTemp);

  Serial.print("      Temp1: ");
  Serial.print(Temp1);
  Serial.print("      Moisture1: ");
  Serial.println(analogRead(1));

  Serial.print("      Temp3: "); 
  Serial.print(Temp3);
  Serial.print("      Moisture3: ");  
  Serial.println(analogRead(2));

  Serial.print("      Temp5: "); 
  Serial.print(Temp5);
  Serial.print("      Moisture5: ");  
  Serial.println(analogRead(3));

  Serial.print("      Temp7: "); 
  Serial.print(Temp7);
  Serial.print("      Moisture7: ");  
  Serial.println(analogRead(4));

  Serial.print("      Temp9: "); 
  Serial.print(Temp9);
  Serial.print("      Moisture9: ");  
  Serial.println(analogRead(5));

  Serial.print("      Temp11: "); 
  Serial.print(Temp11);
  Serial.print("      Moisture11: ");  
  Serial.println(analogRead(6));

  Serial.print("      TempT: "); 
  Serial.print(TempT);
  Serial.print("      MoistureT: ");  
  Serial.println(analogRead(7));
  
  Serial.print("      Average Temp: "); 
  Serial.print(AvgTemp);
  Serial.print("      Average Moisture: ");  
  Serial.println(AvgMoist);
  Serial.println("");

////////////////////////////////////////Write to microSD
//Create data string for microSD file
String dataString = String(RTC.readDateTime())+ "," + String(photocellReading) + "," + String(h) + "," + String(AirTemp) + "," + String(Temp1) + "," + String(analogRead(1)) + "," + String(Temp3) + "," + String(analogRead(2))+ "," + String(Temp5) + "," + String(analogRead(3))+ "," + String(Temp7) + "," + String(analogRead(4))+ "," + String(Temp9) + "," + String(analogRead(5))+ "," + String(Temp11) + "," + String(analogRead(6))+ "," + String(TempT) + "," + String(analogRead(7)) + "," + String(AvgTemp) + "," + String(AvgMoist);

//Let the microSD talk via SPI CS pin
  digitalWrite( SD_CS, LOW );

//Open file and write dataString
  myFile = SD.open(filename, FILE_WRITE);
    myFile.println(dataString);
    myFile.close();
    Serial.println(dataString);

////////////////////////////////////////Send Data to Ubidots

//Prepare JSON payload for Ubidots & get length
  int length = 0;
  String webData1 = "{\"value\":"+ String(AirTemp) +"}";
  String webData2 = "{\"value\":"+ String(AvgTemp) +"}";
  String webData3 = "{\"value\":"+ String(AvgMoist) +"}";
  String webData4 = "{\"value\":"+ String(Temp5) +"}";
  String webData5 = "{\"value\":"+ String(analogRead(7)) +"}";
  String webData6 = "{\"value\":"+ String(photocellReading) +"}";
  String webData7 = "{\"value\":"+ String(h) +"}";

  stringLength1 = webData1.length(); 
  stringLength2 = webData2.length();
  stringLength3 = webData3.length();
  stringLength4 = webData4.length();
  stringLength5 = webData5.length();
  stringLength6 = webData6.length();
  stringLength7 = webData7.length();

//This is used for initial troubleshooting. Uncomment if your upload fails then verify content length here versus what is reported back from the client
  Serial.println(F("Data Length1: "));
  Serial.print(stringLength1);
  Serial.println(F("Data Length2: "));
  Serial.print(stringLength2);
  Serial.println(F("Data Length3: "));
  Serial.print(stringLength3);
  Serial.println(F("Data Length4: "));
  Serial.print(stringLength4);
  Serial.println(F("Data Length5: "));
  Serial.print(stringLength5);
  Serial.println(F("Data Length6: "));
  Serial.print(stringLength6);
  Serial.println(F("Data Length7: "));
  Serial.print(stringLength7);
  Serial.println();
  Serial.println(F("POST /api/v1.6/variables/"));
  Serial.println(F("Host: things.ubidots.com"));
  Serial.println(F("Accept: application/json"));
  Serial.println(F("Content-Type: application/json"));
  Serial.println("Content-Length 1: " + String(stringLength1));
  Serial.println("Content-Length 2: " + String(stringLength2));
  Serial.println("Content-Length 3: " + String(stringLength3));
  Serial.println("Content-Length 4: " + String(stringLength4));
  Serial.println("Content-Length 5: " + String(stringLength5));
  Serial.println("Content-Length 6: " + String(stringLength6));
  Serial.println("Content-Length 7: " + String(stringLength7));
  Serial.println();
  Serial.println(webData1);
  Serial.println(webData2);
  Serial.println(webData3);
  Serial.println(webData4);
  Serial.println(webData5);
  Serial.println(webData6);
  Serial.println(webData7);
  Serial.println();

//CC3000 reboot goto here
  beginning:
  
//Shut up microSD SPI CS pin so WiFi can talk
  digitalWrite( SD_CS, HIGH );
  digitalWrite( WiFi_CS, LOW );

//Check to see if WiFI connection still exists. If not, reconnect
  if (!cc3000.checkConnected()) 
  {
    Serial.println(F("Lost WiFi connection! Attempting to reconnect"));
    cc3000.connectToAP(WLAN_SSID, WLAN_PASS, WLAN_SECURITY);
    Serial.println(F("WiFi connected! 2"));
//Request DHCP - If not resolved in 30 seconds, try to reconnect to AP and start over
    Serial.println(F("Requesting DHCP. 2"));
    while (!cc3000.checkDHCP()) 
    {
      delay(500);
      Serial.print(F("."));
      dhcpTimeout ++;
      if (dhcpTimeout == 60) 
      {
        Serial.println(F(""));
        Serial.println("DHCP Timeout! Rebooting CC3000 and attempting to reconnect.");
        dhcpTimeout = 0;
        cc3000.reboot();
        goto beginning;  
      }
    }
    Serial.println(F(""));
    Serial.println(F("DHCP complete. 2"));
  } 

// Send AirTemp to Ubidots
  Adafruit_CC3000_Client client = cc3000.connectTCP(ip, 80);
  if (client.connected()) 
  {
   Serial.println(F("Connected and sending AirTemp!"));
   client.print(F("POST /api/v1.6/variables/"));
   client.print(idvariable1);
   client.println(F("/values HTTP/1.1"));
   client.println(F("Content-Type: application/json"));
   client.print(F("Content-Length: "));
   client.println(stringLength1);
   client.print(F("X-Auth-Token: "));
   client.println(token);
   client.println(F("Host: things.ubidots.com"));
   client.println();
   client.println(webData1);
   client.println();
   Serial.println(F("AirTemp Sent!"));
   } 
    else 
   {
  Serial.println(F("Connection for AirTemp failed."));    
    return;
   }

// Send Average Soil Temp data to Ubidots
  if (client.connected()) 
  {
   Serial.println(F("Connected and sending Average Soil Temp data!"));
   client.print(F("POST /api/v1.6/variables/"));
   client.print(idvariable2);
   client.println(F("/values HTTP/1.1"));
   client.println(F("Content-Type: application/json"));
   client.print(F("Content-Length: "));
   client.println(stringLength2);
   client.print(F("X-Auth-Token: "));
   client.println(token);
   client.println(F("Host: things.ubidots.com"));
   client.println();
   client.println(webData2);
   client.println();
   Serial.println(F("Average Soil Temp data sent!"));
   } 
    else 
   {
  Serial.println(F("Connection for Average Soil Temp data failed."));    
    return;
   }

// Send Average Soil Moisture Data to Ubidots
  if (client.connected()) 
  {
   Serial.println(F("Connected and sending Average Soil Moisture Data!"));
   client.print(F("POST /api/v1.6/variables/"));
   client.print(idvariable3);
   client.println(F("/values HTTP/1.1"));
   client.println(F("Content-Type: application/json"));
   client.print(F("Content-Length: "));
   client.println(stringLength3);
   client.print(F("X-Auth-Token: "));
   client.println(token);
   client.println(F("Host: things.ubidots.com"));
   client.println();
   client.println(webData3);
   client.println();
   Serial.println(F("Average Soil Moisture Sent!"));
   } 
    else 
   {
  Serial.println(F("Connection for Average Soil Moisture Data failed."));    
    return;
   }   

// Send Tomato Bed Temp Data to Ubidots
  if (client.connected()) 
  {
   Serial.println(F("Connected and sending Tomato Bed Temp Data!"));
   client.print(F("POST /api/v1.6/variables/"));
   client.print(idvariable4);
   client.println(F("/values HTTP/1.1"));
   client.println(F("Content-Type: application/json"));
   client.print(F("Content-Length: "));
   client.println(stringLength4);
   client.print(F("X-Auth-Token: "));
   client.println(token);
   client.println(F("Host: things.ubidots.com"));
   client.println();
   client.println(webData4);
   client.println();
   Serial.println(F("Tomato Bed Temp Data Sent!"));
   } 
    else 
   {
  Serial.println(F("Connection for Tomato Bed Temp Data failed."));    
    return;
   }

// Send Tomato Bed Moisture Data to Ubidots
  if (client.connected()) 
  {
   Serial.println(F("Connected and sending Tomato Bed Moisture Data!"));
   client.print(F("POST /api/v1.6/variables/"));
   client.print(idvariable5);
   client.println(F("/values HTTP/1.1"));
   client.println(F("Content-Type: application/json"));
   client.print(F("Content-Length: "));
   client.println(stringLength5);
   client.print(F("X-Auth-Token: "));
   client.println(token);
   client.println(F("Host: things.ubidots.com"));
   client.println();
   client.println(webData5);
   client.println();
   Serial.println(F("Tomato Bed Moisture Data Sent!"));
   } 
    else 
   {
  Serial.println(F("Connection for Tomato Bed Moisture Data failed."));    
    return;
   }
// Send Light Data to Ubidots
  if (client.connected()) 
  {
   Serial.println(F("Connected and sending Light data!"));
   client.print(F("POST /api/v1.6/variables/"));
   client.print(idvariable6);
   client.println(F("/values HTTP/1.1"));
   client.println(F("Content-Type: application/json"));
   client.print(F("Content-Length: "));
   client.println(stringLength6);
   client.print(F("X-Auth-Token: "));
   client.println(token);
   client.println(F("Host: things.ubidots.com"));
   client.println();
   client.println(webData6);
   client.println();
   Serial.println(F("Light data Sent!"));
   } 
    else 
   {
  Serial.println(F("Connection for Light data failed."));    
    return;
   }
/*
// Send Humidity Data to Ubidots
if (client.connected()) 
  {
   Serial.println(F("Connected and sending Humidity Data!"));
   client.print(F("POST /api/v1.6/variables/"));
   client.print(idvariable7);
   client.println(F("/values HTTP/1.1"));
   client.println(F("Content-Type: application/json"));
   client.print(F("Content-Length: "));
   client.println(stringLength7);
   client.print(F("X-Auth-Token: "));
   client.println(token);
   client.println(F("Host: things.ubidots.com"));
   client.println();
   client.println(webData7);
   client.println();
   Serial.println(F("Humidity Data Sent!"));
   } 
    else 
   {
  Serial.println(F("Connection for Humidity data failed."));    
    return;
   }


//for troubleshooting payload delivery errors to Ubidots  
  Serial.println(F("----------"));
  while (client.connected()) 
  {
    while (client.available()) 
    {
      char c = client.read();
      Serial.print(c);
    }
  }
*/
//Built in delay before shutting off WiFi, otherwise last set of data doesn't send
  delay (6000);  

//Turn off WiFi  
  client.close();
  Serial.println(F(""));
  Serial.println(F("Wifi disconnected..."));

//interrupt WiFi SPI CS pin
  digitalWrite( WiFi_CS, HIGH );

//delay til next read. First value for testing (2 min), second for actual (20 min)
  delay(120000);
  //delay(1200000);
}

//Temp sensorValue function
  float sensorValue (byte deviceAddress[])
  {
  tempC = sensors.getTempC (deviceAddress);
  return tempC;
  }
